package ink.wgink.module.group.controller.api;

import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.group.GroupDTO;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.module.group.pojo.vos.GroupVO;
import ink.wgink.module.group.service.IGroupService;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: GroupController
 * @Description: 组
 * @Author: wenc
 * @Date: 2019/1/6 3:03 PM
 * @Version: 1.0
 **/
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "组信息管理接口")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/group")
public class GroupController extends DefaultBaseController {

    @Autowired
    private IGroupService groupService;

    @ApiOperation(value = "组新增", notes = "组新增接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("savegroup")
    public SuccessResult saveGroup(@RequestBody GroupVO groupVO) {
        Map<String, Object> params = getParams();
        checkParams(groupVO, params);
        return groupService.saveGroup(params);
    }

    @ApiOperation(value = "组删除", notes = "通过id列表批量删除组接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "组ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("removegroup/{ids}")
    public SuccessResult removeGroup(@PathVariable("ids") String ids) {
        Map<String, Object> params = getParams();
        params.put("groupIds", ids);
        return groupService.removeGroup(params);
    }

    @ApiOperation(value = "组修改", notes = "组修改接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "groupId", value = "组ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("updategroup/{groupId}")
    public SuccessResult updateGroup(@PathVariable("groupId") String groupId, @RequestBody GroupVO groupVO) {
        Map<String, Object> params = getParams();
        params.put("groupId", groupId);
        checkParams(groupVO, params);
        return groupService.updateGroup(params);
    }

    /**
     * 参数校验
     *
     * @param groupVO
     * @param params
     * @throws ParamsException
     */
    private void checkParams(GroupVO groupVO, Map<String, Object> params) {
        if (StringUtils.isBlank(groupVO.getGroupParentId())) {
            throw new ParamsException("父ID不能为空");
        }
        params.put("groupParentId", groupVO.getGroupParentId());

        if (StringUtils.isBlank(groupVO.getGroupName())) {
            throw new ParamsException("组名称不能为空");
        }
        params.put("groupName", groupVO.getGroupName());

        params.put("groupSummary", groupVO.getGroupSummary());
    }

    @ApiOperation(value = "组列表", notes = "组列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "groupParentId", value = "组上级ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listgroup/{groupParentId}")
    public List<GroupDTO> listGroup(@PathVariable("groupParentId") String groupParentId) {
        Map<String, Object> params = getParams();
        params.put("groupParentId", groupParentId);
        return groupService.listGroupAllJson(params);
    }

    @ApiOperation(value = "组详情", notes = "组详情接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "groupId", value = "组ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("getgroup/{groupId}")
    public GroupDTO getGroup(@PathVariable("groupId") String groupId) {
        Map<String, Object> params = getParams();
        params.put("groupId", groupId);
        return groupService.getGroup(params);
    }

    @ApiOperation(value = "zTree列表", notes = "zTree列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "父ID", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listztreegroup")
    public List<ZTreeDTO> listZTreeGroup() {
        Map<String, Object> params = requestParams();
        String groupParentId = "0";
        if (!StringUtils.isBlank(params.get("id") == null ? null : params.get("id").toString())) {
            groupParentId = params.get("id").toString();
        }
        params.put("groupParentId", groupParentId);
        return groupService.listZTreeGroup(params);
    }

    @ApiOperation(value = "分页组列表", notes = "分页组列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "parentId", value = "上级ID", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpagegroup")
    public SuccessResultList<List<GroupDTO>> listPageGroup(ListPage page) {
        Map<String, Object> params = requestParams();
        String groupParentId = "0";
        if (!StringUtils.isBlank(params.get("parentId") == null ? null : params.get("parentId").toString())) {
            groupParentId = params.get("parentId").toString();
        }
        params.put("groupParentId", groupParentId);
        page.setParams(params);
        return groupService.listPageGroups(page);
    }

    @ApiOperation(value = "角色组删除", notes = "角色组删除接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "roleId", value = "角色ID", paramType = "path"),
            @ApiImplicitParam(name = "ids", value = "组ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("removerolegroup/{roleId}/{ids}")
    public SuccessResult removeRoleGroup(@PathVariable("roleId") String roleId, @PathVariable("ids") String ids) {
        Map<String, Object> params = getParams();
        params.put("roleId", roleId);
        params.put("groupIds", ids);
        return groupService.removeRoleGroup(params);
    }

    @ApiOperation(value = "通过ID列表获取组列表", notes = "通过ID列表获取用户组接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "groupIds", value = "组ID列表，用下划线分隔，如：1_2_3", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listgroupbyids/{groupIds}")
    public List<GroupDTO> listGroupByIds(@PathVariable("groupIds") String groupIds) {
        Map<String, Object> params = getParams();
        params.put("groupIds", Arrays.asList(groupIds.split("_")));
        return groupService.listGroup(params);
    }

}
