package ink.wgink.module.position.controller.api;

import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.position.PositionDTO;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.module.position.pojo.vos.PositionVO;
import ink.wgink.module.position.service.IPositionService;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: PositionController
 * @Description: 职位
 * @Author: admin
 * @Date: 2019-04-11 22:41:20
 * @Version: 1.0
 **/
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "职位管理接口")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/position")
public class PositionController extends DefaultBaseController {

    @Autowired
    private IPositionService positionService;

    @ApiOperation(value = "职位新增", notes = "职位新增接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("saveposition")
    public SuccessResult savePosition(@RequestBody PositionVO positionVO) {
        Map<String, Object> params = getParams();
        checkParams(positionVO, params);
        return positionService.save(params);
    }

    @ApiOperation(value = "职位删除", notes = "通过id列表批量删除职位接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "职位ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("removeposition/{ids}")
    public SuccessResult removePosition(@PathVariable("ids") String ids) {
        Map<String, Object> params = getParams();
        params.put("positionIds", ids);
        return positionService.remove(params);
    }

    @ApiOperation(value = "职位修改", notes = "职位修改接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "positionId", value = "职位ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("updateposition/{positionId}")
    public SuccessResult updatePosition(@PathVariable("positionId") String positionId, @RequestBody PositionVO positionVO) {
        Map<String, Object> params = getParams();
        params.put("positionId", positionId);
        checkParams(positionVO, params);
        return positionService.update(params);
    }

    /**
     * 校验参数
     *
     * @param positionVO
     * @param params
     */
    private void checkParams(PositionVO positionVO, Map<String, Object> params) {
        if (StringUtils.isBlank(positionVO.getPositionParentId())) {
            throw new ParamsException("父ID不能为空");
        }
        params.put("positionParentId", positionVO.getPositionParentId());

        if (StringUtils.isBlank(positionVO.getPositionName())) {
            throw new ParamsException("职位名称不能为空");
        }
        params.put("positionName", positionVO.getPositionName());

        params.put("positionSummary", positionVO.getPositionSummary());

        params.put("positionNameEn", positionVO.getPositionNameEn());
    }

    @ApiOperation(value = "职位列表", notes = "职位列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "positionParentId", value = "职位上级ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listposition/{positionParentId}")
    public List<PositionDTO> listPosition(@PathVariable("positionParentId") String positionParentId) {
        Map<String, Object> params = getParams();
        params.put("positionParentId", positionParentId);
        return positionService.list(params);
    }

    @ApiOperation(value = "职位详情", notes = "职位详情接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "positionId", value = "职位ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("getposition/{positionId}")
    public PositionDTO getPosition(@PathVariable("positionId") String positionId) {
        Map<String, Object> params = getParams();
        params.put("positionId", positionId);
        return positionService.get(params);
    }

    @ApiOperation(value = "zTree职位列表", notes = "zTree职位列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "父ID", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listztreeposition")
    public List<ZTreeDTO> listZTreePosition() {
        Map<String, Object> params = requestParams();
        String positionParentId = "0";
        if (!StringUtils.isBlank(params.get("id") == null ? null : params.get("id").toString())) {
            positionParentId = params.get("id").toString();
        }
        params.put("positionParentId", positionParentId);
        return positionService.listZTree(params);
    }

    @ApiOperation(value = "分页职位列表", notes = "分页职位列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "parentId", value = "上级ID", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpageposition")
    public SuccessResultList<List<PositionDTO>> listPagePosition(ListPage page) {
        Map<String, Object> params = requestParams();
        String positionParentId = "0";
        if (!StringUtils.isBlank(params.get("parentId") == null ? null : params.get("parentId").toString())) {
            positionParentId = params.get("parentId").toString();
        }
        params.put("positionParentId", positionParentId);
        page.setParams(params);
        return positionService.listPage(page);
    }

}
