package ink.wgink.module.user.service.impl;

import com.alibaba.excel.EasyExcel;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.module.file.excel.error.AbstractErrorExcelHandler;
import ink.wgink.module.file.service.IFileService;
import ink.wgink.module.user.dao.IUserDao;
import ink.wgink.module.user.excel.UserExcel;
import ink.wgink.module.user.excel.UserExcelError;
import ink.wgink.module.user.excel.UserExcelListener;
import ink.wgink.module.user.pojo.vos.RestPasswordVO;
import ink.wgink.module.user.pojo.vos.UpdateUsernameVO;
import ink.wgink.module.user.pojo.vos.UserVO;
import ink.wgink.module.user.service.IUserService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.user.UserDTO;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.pojo.result.UploadExcelResultDTO;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.map.HashMapUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: UserServiceImpl
 * @Description: 用户
 * @Author: WangGeng
 * @Date: 2021/1/24 20:25
 * @Version: 1.0
 **/
@Service
public class UserServiceImpl extends DefaultBaseService implements IUserService {

    @Autowired
    private IUserDao userDao;
    @Autowired
    private PasswordEncoder passwordEncoder;
    @Value("${user.default-password:88888888}")
    private String defaultPassword;
    @Autowired
    private IFileService fileService;

    @Override
    public void save(UserVO userVO) {
        saveUser(userVO, false);
    }

    @Override
    public void register(UserVO userVO) {
        saveUser(userVO, true);
    }

    /**
     * 保存用户
     *
     * @param userVO
     * @param isRegister 是否注册
     */
    private void saveUser(UserVO userVO, boolean isRegister) {
        UserDTO userDTO = getByUsername(userVO.getUserUsername());
        if (userDTO != null) {
            throw new SearchException("用户已经存在");
        }
        userVO.setUserPassword(passwordEncoder.encode(DigestUtils.md5Hex(DigestUtils.md5Hex(DigestUtils.md5Hex(userVO.getUserPassword())))));
        String userId = UUIDUtil.getUUID();
        Map<String, Object> params = HashMapUtil.beanToMap(userVO);
        params.put("userId", userId);
        if (isRegister) {
            setSaveInfoByUserId(params, userId);
        } else {
            setSaveInfo(params);
        }
        userDao.save(params);
    }

    @Override
    public void remove(List<String> ids) {
        Map<String, Object> params = getHashMap(2);
        params.put("userIds", ids);
        setUpdateInfo(params);
        userDao.remove(params);
    }

    @Override
    public void update(String userId, UserVO userVO) {
        if (!StringUtils.isBlank(userVO.getUserPassword())) {
            userVO.setUserPassword(passwordEncoder.encode(DigestUtils.md5Hex(DigestUtils.md5Hex(DigestUtils.md5Hex(userVO.getUserPassword())))));
        }
        Map<String, Object> params = HashMapUtil.beanToMap(userVO);
        setUpdateInfo(params);
        params.put("userId", userId);
        userDao.update(params);
    }

    @Override
    public void resetPassword(String userId, RestPasswordVO restPasswordVO) {
        UserDTO userDTO = get(userId);
        if (userDTO == null) {
            throw new SearchException("修改用户不存在");
        }
        Map<String, Object> params = getHashMap(4);
        params.put("userId", userId);
        params.put("userPassword", passwordEncoder.encode(DigestUtils.md5Hex(DigestUtils.md5Hex(DigestUtils.md5Hex(defaultPassword)))));
        userDao.updatePassword(params);
    }

    @Override
    public void updateUsername(String userId, UpdateUsernameVO updateUsernameVO) {
        UserDTO oldUserDTO = get(userId);
        if (oldUserDTO == null) {
            throw new SearchException("用户不存在");
        }
        Map<String, Object> params = getHashMap(2);
        params.put("userUsername", updateUsernameVO.getUsername());
        UserDTO userDTO = userDao.get(params);
        if (userDTO != null && !StringUtils.equals(userId, userDTO.getUserId())) {
            throw new SearchException("新用户名已经存在");
        }
        params.put("userId", userId);
        userDao.updateUsername(params);
    }

    @Override
    public UploadExcelResultDTO importExcel(MultipartFile excel) throws IOException {
        Map<String, Object> params = getHashMap(16);
        // 错误列表
        List<UserExcelError> userExcelErrors = new ArrayList<>();
        long startTime = System.currentTimeMillis();
        LOG.debug("读取Excel");
        EasyExcel.read(excel.getInputStream(), UserExcel.class, new UserExcelListener() {
            @Override
            public void listUserExcel(List<UserExcel> userExcels) throws Exception {
                for (UserExcel userExcel : userExcels) {
                    // 判断昵称存在
                    String userUsername = userExcel.getUserUsername();
                    String userName = userExcel.getUserName();
                    if (StringUtils.isBlank(userUsername) || StringUtils.isBlank(userName)) {
                        userExcelErrors.add(getUserExcelError(userExcel, "必填值存在空值，导入失败"));
                        continue;
                    }
                    String userPhone = userExcel.getUserPhone();
                    String userEmail = userExcel.getUserEmail();
                    try {
                        // 保存用户
                        UserVO userVO = new UserVO();
                        userVO.setUserUsername(userUsername);
                        userVO.setUserName(userName);
                        userVO.setUserPassword(defaultPassword);
                        userVO.setUserPhone(userPhone);
                        userVO.setUserEmail(userEmail);
                        userVO.setUserType(2);
                        userVO.setUserState(0);
                        saveUser(userVO, false);
                    } catch (Exception e) {
                        userExcelErrors.add(getUserExcelError(userExcel, e.getMessage()));
                    }
                }
            }
        }).headRowNumber(2).sheet().doRead();
        long endTime = System.currentTimeMillis();
        String excelFileId = null;
        if (userExcelErrors.size() > 0) {
            excelFileId = new AbstractErrorExcelHandler<UserExcelError>(fileService) {
                @Override
                public List<List<String>> excelHeaderNames() {
                    return simpleExcelHeader(new String[]{"用户名", "昵称", "手机", "邮箱", "错误原因"});
                }
            }.saveErrorExcel(userExcelErrors);
        }
        return new UploadExcelResultDTO(userExcelErrors.size(), endTime - startTime, excelFileId);
    }

    @Override
    public UserDTO get(String userId) {
        Map<String, Object> params = getHashMap(2);
        params.put("userId", userId);
        return userDao.get(params);
    }

    @Override
    public UserDTO getByUsername(String username) {
        Map<String, Object> params = getHashMap(2);
        params.put("userUsername", username);
        return userDao.get(params);
    }

    @Override
    public List<UserDTO> listByUserIds(List<String> userIds) {
        Map<String, Object> params = getHashMap(2);
        params.put("userIds", userIds);
        return list(params);
    }

    @Override
    public List<UserDTO> listByUsernames(List<String> usernames) {
        Map<String, Object> params = getHashMap(2);
        params.put("userUsernames", usernames);
        return list(params);
    }

    @Override
    public List<UserDTO> list(Map<String, Object> params) {
        return userDao.list(params);
    }

    @Override
    public SuccessResultList<List<UserDTO>> listPage(ListPage page) {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<UserDTO> userDTOs = list(page.getParams());
        PageInfo<UserDTO> pageInfo = new PageInfo<>(userDTOs);
        return new SuccessResultList<>(userDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

    /**
     * Excel导入错误对象
     *
     * @param userExcel
     * @param reason
     * @return
     */
    private UserExcelError getUserExcelError(UserExcel userExcel, String reason) {
        UserExcelError userExcelError = new UserExcelError();
        userExcelError.setUserUsername(userExcel.getUserUsername());
        userExcelError.setUserName(userExcel.getUserName());
        userExcelError.setUserPhone(userExcel.getUserPhone());
        userExcelError.setUserEmail(userExcel.getUserEmail());
        userExcelError.setReason(reason);
        return userExcelError;
    }

}
